/* decode.c		read codes from infile and construct array
 *
 * Programmer: R. White		Date: 9 May 1991
 *
 * Modified by J. Doggett, 7 December 1993
 *         to add <stdlib.h> include for standard function prototypes
 *         to add <string.h> include for string function prototypes
 *
 * Modified by J. Doggett, 15 December 1993
 *         put in char casts for code_magic because in general, a hex
 *         value won't fit in a char, so the acc compiler would complain
 *         about it. It's happy now.
 */

/******************************************************************************
* Copyright (c) 1993, 1994, Association of Universities for Research in       *
* Astronomy, Inc.  All Rights Reserved.                                       *
* Produced under National Aeronautics and Space Administration grant          *
* NAG W-2166.                                                                 *
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "qfile.h"

char code_magic[2] = { (char)0xDD, (char)0x99 };

extern void dodecode();

extern void
decode(infile,a,nx,ny,scale)
QFILE *infile;							/* input file (already opened)		*/
int **a;								/* address of output array [nx][ny]	*/
int *nx,*ny;							/* size of output array				*/
int *scale;								/* scale factor for digitization	*/
{
int i, nel, sumall;
unsigned char nbitplanes[3];
char tmagic[2];

	/*
	 * read magic value in first two bytes
	 */
	qread(infile, tmagic, sizeof(tmagic));
	if (memcmp(tmagic,code_magic,sizeof(code_magic)) != 0) {
		fprintf(stderr, "bad file format\n");
		exit(-1);
	}
	*nx =readint(infile);				/* x size of image					*/
	*ny =readint(infile);				/* y size of image					*/
	*scale=readint(infile);				/* scale factor for digitization	*/
	/*
	 * allocate memory for array
	 */
	nel = (*nx) * (*ny);
	*a = (int *) malloc(nel*sizeof(int));
	if (*a == (int *) NULL) {
		fprintf(stderr, "decode: insufficient memory\n");
		exit(-1);
	}
	sumall=readint(infile);							/* sum of all pixels	*/
	qread(infile, nbitplanes, sizeof(nbitplanes));	/* # bits in quadrant	*/
	dodecode(infile, *a, *nx, *ny, nbitplanes);
	/*
	 * put sum of all pixels back into pixel 0
	 */
	(*a)[0] = sumall;
}
